from package.data.project import get_project_name, update_project_status
from package.data.record import get_recent_pending_record_by_project, update_stop_record
from package.entity.record import Record
from package.utils.utils import get_config
from datetime import datetime
import typer
import subprocess


def stop(project_name):
    project = get_project_name(project_name=project_name)
    if not project:
        raise ValueError("找不到项目，请重新输入")
    record_ = get_recent_pending_record_by_project(
        Record(id=None, project_id=project.id)
    )

    if record_:
        start_time = record_.start
    else:
        typer.secho("找不到正在进行的记录")
        return
    if not start_time:
        raise ValueError("找不到开始时间，请重新输入")
    stop_time = int(datetime.today().timestamp())
    duration_time = stop_time - start_time
    record = Record(
        id=None, stop=stop_time, duration=duration_time, project_id=project.id
    )
    update_stop_record(record)
    project.project_status = 0
    update_project_status(project=project)
    config = get_config()
    if config["enable_timew"]:
        params = ["timew", "stop", project_name]
        subprocess.run(params)



from dataclasses import dataclass


@dataclass
class Record:
    id: int
    start: int = -1
    stop: int = -1
    duration: int = -1
    task_desc: str = ""
    extra_id: str = ""
    project_id: int = -1



from package.entity.project import Project
from package.data.project import add_project as add_project_data
from package.data.init import ensure_db_ready


@ensure_db_ready
def add_project(project_name, total, rate):
    print("hello", total, rate)
    project = Project(
        None, project_name=project_name, fatigue_rate=rate, fatigue_total=total
    )

    add_project_data(project)



from package.data.project import get_all_projects
import typer


def complete_project_name(ctx: typer.Context, param, incomplete: str):
    text = incomplete
    projects = get_all_projects()
    choices = [
        p.project_name
        for p in projects
        if isinstance(p.project_name, str) and p.project_name.startswith(text)
    ]

    return choices



from package.data.project import delete_project as delete_project_data
from package.data.init import ensure_db_ready


@ensure_db_ready
def delete_project(project_name):
    delete_project_data(project_name)



import sqlite3
import functools
import os
from pathlib import Path
import json


_is_initialized = False


DB_DIR = str(Path.home() / ".config" / "mana" / "data")
DB_PATH = os.path.join(DB_DIR, "fatigue.db")
CONFIG_PATH = os.path.join(DB_DIR, "config.json")


def _init_db():
    global _is_initialized

    if _is_initialized:
        return
    conn = get_conn()
    cursor = conn.cursor()
    cursor.executescript(
        """
        CREATE TABLE IF NOT EXISTS records (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            start INTEGER,
            stop INTEGER,
            duration INTEGER,   
            task_desc TEXT,
            extra_id TEXT,
            project_id INTEGER
        );
        
        CREATE TABLE IF NOT EXISTS projects (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            project_name TEXT UNIQUE,
            fatigue_rate REAL DEFAULT 1.0,
            fatigue_total INTEGER DEFAULT 156,
            project_status INTEGER DEFAULT 0
        );
    """
    )

    conn.commit()
    conn.close()
    _is_initialized = True


def _create_config():
    if os.path.exists(CONFIG_PATH):
        return
    else:
        with open(CONFIG_PATH, "w", encoding="utf-8") as f:
            default_config = {"enable_timew": True}
            json.dump(default_config, f, indent=4)


def get_conn():
    if not os.path.exists(DB_DIR):
        os.makedirs(DB_DIR, exist_ok=True)

    conn = sqlite3.connect(DB_PATH)
    conn.row_factory = sqlite3.Row
    return conn


def ensure_db_ready(func):
    @functools.wraps(func)
    def wrapper(*args, **kargs):
        _init_db()
        _create_config()
        return func(*args, **kargs)

    return wrapper



from package.data.init import ensure_db_ready, get_conn
from package.entity.record import Record
from package.entity.project import Project
from datetime import datetime
import time


@ensure_db_ready
def add_record(record: Record):
    sql = "insert into records(start,stop,duration,task_desc,extra_id,project_id) values(?,?,?,?,?,?)"
    with get_conn() as conn:
        cursor = conn.cursor()
        cursor.execute(
            sql,
            (
                record.start,
                record.stop,
                record.duration,
                record.task_desc,
                record.extra_id,
                record.project_id,
            ),
        )
        conn.commit()


@ensure_db_ready
def update_stop_record(record: Record):
    sql = (
        "update records set stop = ? , duration = ? where project_id = ? and stop = -1"
    )
    with get_conn() as conn:
        cursor = conn.cursor()
        cursor.execute(
            sql,
            (
                record.stop,
                record.duration,
                record.project_id,
            ),
        )
        conn.commit()


@ensure_db_ready
def get_recent_pending_record_by_project(record: Record):
    sql = (
        "select id,start,stop,task_desc from records where stop = -1 and project_id = ?"
    )
    with get_conn() as conn:
        cursor = conn.cursor()
        cursor.execute(
            sql,
            (record.project_id,),
        )
        row = cursor.fetchone()

        if not row:
            return None

        return Record(**dict(row))


@ensure_db_ready
def get_recent_record_by_project(record: Record):
    sql = "select id,start,stop,task_desc from records where project_id = ? ORDER BY start DESC LIMIT 1"
    with get_conn() as conn:
        cursor = conn.cursor()
        cursor.execute(
            sql,
            (record.project_id,),
        )
        row = cursor.fetchone()

        if not row:
            return None

        return Record(**dict(row))


@ensure_db_ready
def get_today_accumulated_fatigue(project_id=None):
    # 截止今天0点的时间戳
    today = datetime.now().replace(hour=4, minute=0, second=0, microsecond=0)
    now_ts = int(time.time())
    today_ts = int(today.timestamp())

    sql = """
        SELECT
        p.id,
        p.project_name,
        p.fatigue_total,
        SUM(
            CASE
                WHEN r.stop = -1 then (? - start)
                ELSE r.duration
            END * p.fatigue_rate / 60
        ) AS used_fatigue,
        p.project_status
        FROM projects p
        LEFT JOIN records r ON r.project_id = p.id AND r.start >= ?

    """
    params = [now_ts, today_ts]
    if project_id:
        sql = sql + " WHERE project_id = ?"
        params.append(project_id)

    sql += " GROUP BY p.id"
    with get_conn() as conn:
        cursor = conn.cursor()
        cursor.execute(
            sql,
            tuple(params),
        )

        if project_id:
            row = cursor.fetchone()
            if row is not None:
                total_duration = row[3] if row[3] is not None else 0
                return total_duration
            else:
                return 0
        else:
            rows = cursor.fetchall()
            results = []
            for r in rows:
                results.append(
                    {
                        "id": r[0],
                        "name": r[1],
                        "total": r[2],
                        "used": r[3] if r[3] is not None else 0,
                        "status": r[4],
                    }
                )
            return results



from package.data.project import get_project_name, add_project, update_project_status
from package.data.record import add_record, get_recent_pending_record_by_project
from package.entity.project import Project
from package.entity.record import Record
from package.utils.utils import get_config
from datetime import datetime
import typer
import subprocess


def start(task_desc, project_name, extra_id):
    p = get_project_name(project_name)
    if not p:
        p = Project(None, project_name=project_name, project_status=1)
        p_id = add_project(p)
    else:
        p_id = p.id

    record = get_recent_pending_record_by_project(Record(id=None, project_id=p_id))
    if record:
        typer.secho("项目已经进行，请先停止", fg=typer.colors.RED, err=True)
        return
    start_time = int(datetime.today().timestamp())
    record = Record(
        None, start=start_time, task_desc=task_desc, project_id=p_id, extra_id=extra_id
    )
    add_record(record=record)
    p.project_status = 1
    update_project_status(project=p)

    config = get_config()
    if config["enable_timew"]:
        params = ["timew", "start", project_name]
        subprocess.run(params)



from package.cli.app import app
import typer


def main():
    try:
        app()
    except Exception as e:
        typer.secho(f"出现异常: {e}", fg=typer.colors.RED, err=True)


if __name__ == "__main__":
    main()



from pathlib import Path
import os
import json

CONFIG_DIR = str(Path.home() / ".config" / "mana" / "data")
CONFIG_PATH = os.path.join(CONFIG_DIR, "config.json")


def get_config():
    with open(CONFIG_PATH, "r", encoding="utf-8") as f:
        return json.load(f)
